# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.logpush import edge_create_params
from ...types.logpush.instant_logpush_job import InstantLogpushJob

__all__ = ["EdgeResource", "AsyncEdgeResource"]


class EdgeResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EdgeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EdgeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EdgeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EdgeResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        fields: str | Omit = omit,
        filter: str | Omit = omit,
        sample: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[InstantLogpushJob]:
        """
        Creates a new Instant Logs job for a zone.

        Args:
          zone_id: Identifier.

          fields: Comma-separated list of fields.

          filter: Filters to drill down into specific events.

          sample:
              The sample parameter is the sample rate of the records set by the client:
              "sample": 1 is 100% of records "sample": 10 is 10% and so on.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/logpush/edge/jobs", zone_id=zone_id),
            body=maybe_transform(
                {
                    "fields": fields,
                    "filter": filter,
                    "sample": sample,
                },
                edge_create_params.EdgeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[InstantLogpushJob]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[InstantLogpushJob]], ResultWrapper[InstantLogpushJob]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Optional[InstantLogpushJob]]:
        """
        Lists Instant Logs jobs for a zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/logpush/edge/jobs", zone_id=zone_id),
            page=SyncSinglePage[Optional[InstantLogpushJob]],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=InstantLogpushJob,
        )


class AsyncEdgeResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEdgeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEdgeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEdgeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEdgeResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        fields: str | Omit = omit,
        filter: str | Omit = omit,
        sample: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[InstantLogpushJob]:
        """
        Creates a new Instant Logs job for a zone.

        Args:
          zone_id: Identifier.

          fields: Comma-separated list of fields.

          filter: Filters to drill down into specific events.

          sample:
              The sample parameter is the sample rate of the records set by the client:
              "sample": 1 is 100% of records "sample": 10 is 10% and so on.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/logpush/edge/jobs", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "fields": fields,
                    "filter": filter,
                    "sample": sample,
                },
                edge_create_params.EdgeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[InstantLogpushJob]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[InstantLogpushJob]], ResultWrapper[InstantLogpushJob]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Optional[InstantLogpushJob], AsyncSinglePage[Optional[InstantLogpushJob]]]:
        """
        Lists Instant Logs jobs for a zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/logpush/edge/jobs", zone_id=zone_id),
            page=AsyncSinglePage[Optional[InstantLogpushJob]],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=InstantLogpushJob,
        )


class EdgeResourceWithRawResponse:
    def __init__(self, edge: EdgeResource) -> None:
        self._edge = edge

        self.create = to_raw_response_wrapper(
            edge.create,
        )
        self.get = to_raw_response_wrapper(
            edge.get,
        )


class AsyncEdgeResourceWithRawResponse:
    def __init__(self, edge: AsyncEdgeResource) -> None:
        self._edge = edge

        self.create = async_to_raw_response_wrapper(
            edge.create,
        )
        self.get = async_to_raw_response_wrapper(
            edge.get,
        )


class EdgeResourceWithStreamingResponse:
    def __init__(self, edge: EdgeResource) -> None:
        self._edge = edge

        self.create = to_streamed_response_wrapper(
            edge.create,
        )
        self.get = to_streamed_response_wrapper(
            edge.get,
        )


class AsyncEdgeResourceWithStreamingResponse:
    def __init__(self, edge: AsyncEdgeResource) -> None:
        self._edge = edge

        self.create = async_to_streamed_response_wrapper(
            edge.create,
        )
        self.get = async_to_streamed_response_wrapper(
            edge.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["MessageBulkPushParams", "Message", "MessageMqQueueMessageText", "MessageMqQueueMessageJson"]


class MessageBulkPushParams(TypedDict, total=False):
    account_id: Required[str]
    """A Resource identifier."""

    delay_seconds: float
    """The number of seconds to wait for attempting to deliver this batch to consumers"""

    messages: Iterable[Message]


class MessageMqQueueMessageText(TypedDict, total=False):
    body: str

    content_type: Literal["text"]

    delay_seconds: float
    """
    The number of seconds to wait for attempting to deliver this message to
    consumers
    """


class MessageMqQueueMessageJson(TypedDict, total=False):
    body: object

    content_type: Literal["json"]

    delay_seconds: float
    """
    The number of seconds to wait for attempting to deliver this message to
    consumers
    """


Message: TypeAlias = Union[MessageMqQueueMessageText, MessageMqQueueMessageJson]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from .asns import (
    ASNsResource,
    AsyncASNsResource,
    ASNsResourceWithRawResponse,
    AsyncASNsResourceWithRawResponse,
    ASNsResourceWithStreamingResponse,
    AsyncASNsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from .locations import (
    LocationsResource,
    AsyncLocationsResource,
    LocationsResourceWithRawResponse,
    AsyncLocationsResourceWithRawResponse,
    LocationsResourceWithStreamingResponse,
    AsyncLocationsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....types.radar import entity_get_params
from ...._base_client import make_request_options
from ....types.radar.entity_get_response import EntityGetResponse

__all__ = ["EntitiesResource", "AsyncEntitiesResource"]


class EntitiesResource(SyncAPIResource):
    @cached_property
    def asns(self) -> ASNsResource:
        return ASNsResource(self._client)

    @cached_property
    def locations(self) -> LocationsResource:
        return LocationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> EntitiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EntitiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EntitiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EntitiesResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        ip: str,
        format: Literal["JSON", "CSV"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EntityGetResponse:
        """
        Retrieves IP address information.

        Args:
          ip: IP address.

          format: Format in which results will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/entities/ip",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ip": ip,
                        "format": format,
                    },
                    entity_get_params.EntityGetParams,
                ),
                post_parser=ResultWrapper[EntityGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[EntityGetResponse], ResultWrapper[EntityGetResponse]),
        )


class AsyncEntitiesResource(AsyncAPIResource):
    @cached_property
    def asns(self) -> AsyncASNsResource:
        return AsyncASNsResource(self._client)

    @cached_property
    def locations(self) -> AsyncLocationsResource:
        return AsyncLocationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEntitiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEntitiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEntitiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEntitiesResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        ip: str,
        format: Literal["JSON", "CSV"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EntityGetResponse:
        """
        Retrieves IP address information.

        Args:
          ip: IP address.

          format: Format in which results will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/entities/ip",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ip": ip,
                        "format": format,
                    },
                    entity_get_params.EntityGetParams,
                ),
                post_parser=ResultWrapper[EntityGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[EntityGetResponse], ResultWrapper[EntityGetResponse]),
        )


class EntitiesResourceWithRawResponse:
    def __init__(self, entities: EntitiesResource) -> None:
        self._entities = entities

        self.get = to_raw_response_wrapper(
            entities.get,
        )

    @cached_property
    def asns(self) -> ASNsResourceWithRawResponse:
        return ASNsResourceWithRawResponse(self._entities.asns)

    @cached_property
    def locations(self) -> LocationsResourceWithRawResponse:
        return LocationsResourceWithRawResponse(self._entities.locations)


class AsyncEntitiesResourceWithRawResponse:
    def __init__(self, entities: AsyncEntitiesResource) -> None:
        self._entities = entities

        self.get = async_to_raw_response_wrapper(
            entities.get,
        )

    @cached_property
    def asns(self) -> AsyncASNsResourceWithRawResponse:
        return AsyncASNsResourceWithRawResponse(self._entities.asns)

    @cached_property
    def locations(self) -> AsyncLocationsResourceWithRawResponse:
        return AsyncLocationsResourceWithRawResponse(self._entities.locations)


class EntitiesResourceWithStreamingResponse:
    def __init__(self, entities: EntitiesResource) -> None:
        self._entities = entities

        self.get = to_streamed_response_wrapper(
            entities.get,
        )

    @cached_property
    def asns(self) -> ASNsResourceWithStreamingResponse:
        return ASNsResourceWithStreamingResponse(self._entities.asns)

    @cached_property
    def locations(self) -> LocationsResourceWithStreamingResponse:
        return LocationsResourceWithStreamingResponse(self._entities.locations)


class AsyncEntitiesResourceWithStreamingResponse:
    def __init__(self, entities: AsyncEntitiesResource) -> None:
        self._entities = entities

        self.get = async_to_streamed_response_wrapper(
            entities.get,
        )

    @cached_property
    def asns(self) -> AsyncASNsResourceWithStreamingResponse:
        return AsyncASNsResourceWithStreamingResponse(self._entities.asns)

    @cached_property
    def locations(self) -> AsyncLocationsResourceWithStreamingResponse:
        return AsyncLocationsResourceWithStreamingResponse(self._entities.locations)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AccessRuleIPConfiguration"]


class AccessRuleIPConfiguration(BaseModel):
    target: Optional[Literal["ip"]] = None
    """The configuration target.

    You must set the target to `ip` when specifying an IP address in the rule.
    """

    value: Optional[str] = None
    """The IP address to match.

    This address will be compared to the IP address of incoming requests.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["IPGeolocationParam"]


class IPGeolocationParam(TypedDict, total=False):
    id: Literal["ip_geolocation"]
    """
    Cloudflare adds a CF-IPCountry HTTP header containing the country code that
    corresponds to the visitor.
    """

    value: Literal["on", "off"]
    """The status of adding the IP Geolocation Header."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .mcp import (
    McpResource,
    AsyncMcpResource,
    McpResourceWithRawResponse,
    AsyncMcpResourceWithRawResponse,
    McpResourceWithStreamingResponse,
    AsyncMcpResourceWithStreamingResponse,
)
from .portals import (
    PortalsResource,
    AsyncPortalsResource,
    PortalsResourceWithRawResponse,
    AsyncPortalsResourceWithRawResponse,
    PortalsResourceWithStreamingResponse,
    AsyncPortalsResourceWithStreamingResponse,
)
from .servers import (
    ServersResource,
    AsyncServersResource,
    ServersResourceWithRawResponse,
    AsyncServersResourceWithRawResponse,
    ServersResourceWithStreamingResponse,
    AsyncServersResourceWithStreamingResponse,
)

__all__ = [
    "PortalsResource",
    "AsyncPortalsResource",
    "PortalsResourceWithRawResponse",
    "AsyncPortalsResourceWithRawResponse",
    "PortalsResourceWithStreamingResponse",
    "AsyncPortalsResourceWithStreamingResponse",
    "ServersResource",
    "AsyncServersResource",
    "ServersResourceWithRawResponse",
    "AsyncServersResourceWithRawResponse",
    "ServersResourceWithStreamingResponse",
    "AsyncServersResourceWithStreamingResponse",
    "McpResource",
    "AsyncMcpResource",
    "McpResourceWithRawResponse",
    "AsyncMcpResourceWithRawResponse",
    "McpResourceWithStreamingResponse",
    "AsyncMcpResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .message import Message as Message
from .settings import Settings as Settings
from .configuration import Configuration as Configuration
from .label_list_params import LabelListParams as LabelListParams
from .old_public_schema import OldPublicSchema as OldPublicSchema
from .schema_list_params import SchemaListParams as SchemaListParams
from .discovery_operation import DiscoveryOperation as DiscoveryOperation
from .label_list_response import LabelListResponse as LabelListResponse
from .operation_get_params import OperationGetParams as OperationGetParams
from .schema_list_response import SchemaListResponse as SchemaListResponse
from .operation_list_params import OperationListParams as OperationListParams
from .discovery_get_response import DiscoveryGetResponse as DiscoveryGetResponse
from .operation_get_response import OperationGetResponse as OperationGetResponse
from .user_schema_get_params import UserSchemaGetParams as UserSchemaGetParams
from .operation_create_params import OperationCreateParams as OperationCreateParams
from .operation_list_response import OperationListResponse as OperationListResponse
from .user_schema_edit_params import UserSchemaEditParams as UserSchemaEditParams
from .user_schema_list_params import UserSchemaListParams as UserSchemaListParams
from .configuration_get_params import ConfigurationGetParams as ConfigurationGetParams
from .operation_create_response import OperationCreateResponse as OperationCreateResponse
from .operation_delete_response import OperationDeleteResponse as OperationDeleteResponse
from .user_schema_create_params import UserSchemaCreateParams as UserSchemaCreateParams
from .configuration_update_params import ConfigurationUpdateParams as ConfigurationUpdateParams
from .user_schema_create_response import UserSchemaCreateResponse as UserSchemaCreateResponse
from .user_schema_delete_response import UserSchemaDeleteResponse as UserSchemaDeleteResponse
from .operation_bulk_create_params import OperationBulkCreateParams as OperationBulkCreateParams
from .operation_bulk_create_response import OperationBulkCreateResponse as OperationBulkCreateResponse
from .operation_bulk_delete_response import OperationBulkDeleteResponse as OperationBulkDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .secret_edit_params import SecretEditParams as SecretEditParams
from .secret_list_params import SecretListParams as SecretListParams
from .secret_get_response import SecretGetResponse as SecretGetResponse
from .secret_create_params import SecretCreateParams as SecretCreateParams
from .secret_edit_response import SecretEditResponse as SecretEditResponse
from .secret_list_response import SecretListResponse as SecretListResponse
from .secret_create_response import SecretCreateResponse as SecretCreateResponse
from .secret_duplicate_params import SecretDuplicateParams as SecretDuplicateParams
from .secret_duplicate_response import SecretDuplicateResponse as SecretDuplicateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from .snapshots import (
    SnapshotsResource,
    AsyncSnapshotsResource,
    SnapshotsResourceWithRawResponse,
    AsyncSnapshotsResourceWithRawResponse,
    SnapshotsResourceWithStreamingResponse,
    AsyncSnapshotsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from .permissions import (
    PermissionsResource,
    AsyncPermissionsResource,
    PermissionsResourceWithRawResponse,
    AsyncPermissionsResourceWithRawResponse,
    PermissionsResourceWithStreamingResponse,
    AsyncPermissionsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ....types.intel import indicator_feed_create_params, indicator_feed_update_params
from ...._base_client import AsyncPaginator, make_request_options
from ....types.intel.indicator_feed_get_response import IndicatorFeedGetResponse
from ....types.intel.indicator_feed_list_response import IndicatorFeedListResponse
from ....types.intel.indicator_feed_create_response import IndicatorFeedCreateResponse
from ....types.intel.indicator_feed_update_response import IndicatorFeedUpdateResponse

__all__ = ["IndicatorFeedsResource", "AsyncIndicatorFeedsResource"]


class IndicatorFeedsResource(SyncAPIResource):
    @cached_property
    def snapshots(self) -> SnapshotsResource:
        return SnapshotsResource(self._client)

    @cached_property
    def permissions(self) -> PermissionsResource:
        return PermissionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> IndicatorFeedsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IndicatorFeedsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IndicatorFeedsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IndicatorFeedsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        description: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndicatorFeedCreateResponse]:
        """
        Creates a new custom threat indicator feed for sharing threat intelligence data.

        Args:
          account_id: Identifier

          description: The description of the example test

          name: The name of the indicator feed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/intel/indicator-feeds", account_id=account_id),
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                indicator_feed_create_params.IndicatorFeedCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndicatorFeedCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndicatorFeedCreateResponse]], ResultWrapper[IndicatorFeedCreateResponse]),
        )

    def update(
        self,
        feed_id: int,
        *,
        account_id: str,
        description: str | Omit = omit,
        is_attributable: bool | Omit = omit,
        is_downloadable: bool | Omit = omit,
        is_public: bool | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndicatorFeedUpdateResponse]:
        """
        Revises details for a specific custom threat indicator feed.

        Args:
          account_id: Identifier

          feed_id: Indicator feed ID

          description: The new description of the feed

          is_attributable: The new is_attributable value of the feed

          is_downloadable: The new is_downloadable value of the feed

          is_public: The new is_public value of the feed

          name: The new name of the feed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/intel/indicator-feeds/{feed_id}", account_id=account_id, feed_id=feed_id
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "is_attributable": is_attributable,
                    "is_downloadable": is_downloadable,
                    "is_public": is_public,
                    "name": name,
                },
                indicator_feed_update_params.IndicatorFeedUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndicatorFeedUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndicatorFeedUpdateResponse]], ResultWrapper[IndicatorFeedUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[IndicatorFeedListResponse]:
        """
        Retrieves details for all accessible custom threat indicator feeds.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/intel/indicator-feeds", account_id=account_id),
            page=SyncSinglePage[IndicatorFeedListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=IndicatorFeedListResponse,
        )

    def data(
        self,
        feed_id: int,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Retrieves the raw data entries in a custom threat indicator feed.

        Args:
          account_id: Identifier

          feed_id: Indicator feed ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "text/csv", **(extra_headers or {})}
        return self._get(
            path_template(
                "/accounts/{account_id}/intel/indicator-feeds/{feed_id}/data", account_id=account_id, feed_id=feed_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    def get(
        self,
        feed_id: int,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndicatorFeedGetResponse]:
        """
        Retrieves details for a specific custom threat indicator feed.

        Args:
          account_id: Identifier

          feed_id: Indicator feed ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/intel/indicator-feeds/{feed_id}", account_id=account_id, feed_id=feed_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndicatorFeedGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndicatorFeedGetResponse]], ResultWrapper[IndicatorFeedGetResponse]),
        )


class AsyncIndicatorFeedsResource(AsyncAPIResource):
    @cached_property
    def snapshots(self) -> AsyncSnapshotsResource:
        return AsyncSnapshotsResource(self._client)

    @cached_property
    def permissions(self) -> AsyncPermissionsResource:
        return AsyncPermissionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncIndicatorFeedsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIndicatorFeedsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIndicatorFeedsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIndicatorFeedsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        description: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndicatorFeedCreateResponse]:
        """
        Creates a new custom threat indicator feed for sharing threat intelligence data.

        Args:
          account_id: Identifier

          description: The description of the example test

          name: The name of the indicator feed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/intel/indicator-feeds", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                indicator_feed_create_params.IndicatorFeedCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndicatorFeedCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndicatorFeedCreateResponse]], ResultWrapper[IndicatorFeedCreateResponse]),
        )

    async def update(
        self,
        feed_id: int,
        *,
        account_id: str,
        description: str | Omit = omit,
        is_attributable: bool | Omit = omit,
        is_downloadable: bool | Omit = omit,
        is_public: bool | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndicatorFeedUpdateResponse]:
        """
        Revises details for a specific custom threat indicator feed.

        Args:
          account_id: Identifier

          feed_id: Indicator feed ID

          description: The new description of the feed

          is_attributable: The new is_attributable value of the feed

          is_downloadable: The new is_downloadable value of the feed

          is_public: The new is_public value of the feed

          name: The new name of the feed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/intel/indicator-feeds/{feed_id}", account_id=account_id, feed_id=feed_id
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "is_attributable": is_attributable,
                    "is_downloadable": is_downloadable,
                    "is_public": is_public,
                    "name": name,
                },
                indicator_feed_update_params.IndicatorFeedUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndicatorFeedUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndicatorFeedUpdateResponse]], ResultWrapper[IndicatorFeedUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[IndicatorFeedListResponse, AsyncSinglePage[IndicatorFeedListResponse]]:
        """
        Retrieves details for all accessible custom threat indicator feeds.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/intel/indicator-feeds", account_id=account_id),
            page=AsyncSinglePage[IndicatorFeedListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=IndicatorFeedListResponse,
        )

    async def data(
        self,
        feed_id: int,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Retrieves the raw data entries in a custom threat indicator feed.

        Args:
          account_id: Identifier

          feed_id: Indicator feed ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "text/csv", **(extra_headers or {})}
        return await self._get(
            path_template(
                "/accounts/{account_id}/intel/indicator-feeds/{feed_id}/data", account_id=account_id, feed_id=feed_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    async def get(
        self,
        feed_id: int,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndicatorFeedGetResponse]:
        """
        Retrieves details for a specific custom threat indicator feed.

        Args:
          account_id: Identifier

          feed_id: Indicator feed ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/intel/indicator-feeds/{feed_id}", account_id=account_id, feed_id=feed_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndicatorFeedGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndicatorFeedGetResponse]], ResultWrapper[IndicatorFeedGetResponse]),
        )


class IndicatorFeedsResourceWithRawResponse:
    def __init__(self, indicator_feeds: IndicatorFeedsResource) -> None:
        self._indicator_feeds = indicator_feeds

        self.create = to_raw_response_wrapper(
            indicator_feeds.create,
        )
        self.update = to_raw_response_wrapper(
            indicator_feeds.update,
        )
        self.list = to_raw_response_wrapper(
            indicator_feeds.list,
        )
        self.data = to_raw_response_wrapper(
            indicator_feeds.data,
        )
        self.get = to_raw_response_wrapper(
            indicator_feeds.get,
        )

    @cached_property
    def snapshots(self) -> SnapshotsResourceWithRawResponse:
        return SnapshotsResourceWithRawResponse(self._indicator_feeds.snapshots)

    @cached_property
    def permissions(self) -> PermissionsResourceWithRawResponse:
        return PermissionsResourceWithRawResponse(self._indicator_feeds.permissions)


class AsyncIndicatorFeedsResourceWithRawResponse:
    def __init__(self, indicator_feeds: AsyncIndicatorFeedsResource) -> None:
        self._indicator_feeds = indicator_feeds

        self.create = async_to_raw_response_wrapper(
            indicator_feeds.create,
        )
        self.update = async_to_raw_response_wrapper(
            indicator_feeds.update,
        )
        self.list = async_to_raw_response_wrapper(
            indicator_feeds.list,
        )
        self.data = async_to_raw_response_wrapper(
            indicator_feeds.data,
        )
        self.get = async_to_raw_response_wrapper(
            indicator_feeds.get,
        )

    @cached_property
    def snapshots(self) -> AsyncSnapshotsResourceWithRawResponse:
        return AsyncSnapshotsResourceWithRawResponse(self._indicator_feeds.snapshots)

    @cached_property
    def permissions(self) -> AsyncPermissionsResourceWithRawResponse:
        return AsyncPermissionsResourceWithRawResponse(self._indicator_feeds.permissions)


class IndicatorFeedsResourceWithStreamingResponse:
    def __init__(self, indicator_feeds: IndicatorFeedsResource) -> None:
        self._indicator_feeds = indicator_feeds

        self.create = to_streamed_response_wrapper(
            indicator_feeds.create,
        )
        self.update = to_streamed_response_wrapper(
            indicator_feeds.update,
        )
        self.list = to_streamed_response_wrapper(
            indicator_feeds.list,
        )
        self.data = to_streamed_response_wrapper(
            indicator_feeds.data,
        )
        self.get = to_streamed_response_wrapper(
            indicator_feeds.get,
        )

    @cached_property
    def snapshots(self) -> SnapshotsResourceWithStreamingResponse:
        return SnapshotsResourceWithStreamingResponse(self._indicator_feeds.snapshots)

    @cached_property
    def permissions(self) -> PermissionsResourceWithStreamingResponse:
        return PermissionsResourceWithStreamingResponse(self._indicator_feeds.permissions)


class AsyncIndicatorFeedsResourceWithStreamingResponse:
    def __init__(self, indicator_feeds: AsyncIndicatorFeedsResource) -> None:
        self._indicator_feeds = indicator_feeds

        self.create = async_to_streamed_response_wrapper(
            indicator_feeds.create,
        )
        self.update = async_to_streamed_response_wrapper(
            indicator_feeds.update,
        )
        self.list = async_to_streamed_response_wrapper(
            indicator_feeds.list,
        )
        self.data = async_to_streamed_response_wrapper(
            indicator_feeds.data,
        )
        self.get = async_to_streamed_response_wrapper(
            indicator_feeds.get,
        )

    @cached_property
    def snapshots(self) -> AsyncSnapshotsResourceWithStreamingResponse:
        return AsyncSnapshotsResourceWithStreamingResponse(self._indicator_feeds.snapshots)

    @cached_property
    def permissions(self) -> AsyncPermissionsResourceWithStreamingResponse:
        return AsyncPermissionsResourceWithStreamingResponse(self._indicator_feeds.permissions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["RoutePfx2asParams"]


class RoutePfx2asParams(TypedDict, total=False):
    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    longest_prefix_match: Annotated[bool, PropertyInfo(alias="longestPrefixMatch")]
    """Return only results with the longest prefix match for the given prefix.

    For example, specify a /32 prefix to lookup the origin ASN for an IPv4 address.
    """

    origin: int
    """Lookup prefixes originated by the given ASN."""

    prefix: str

    rpki_status: Annotated[Literal["VALID", "INVALID", "UNKNOWN"], PropertyInfo(alias="rpkiStatus")]
    """Return only results with matching rpki status: valid, invalid or unknown."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .dataset import Dataset
from ...._models import BaseModel

__all__ = ["DatasetCreation"]


class DatasetCreation(BaseModel):
    dataset: Dataset

    encoding_version: int
    """Encoding version to use for dataset."""

    max_cells: int

    version: int
    """The version to use when uploading the dataset."""

    secret: Optional[str] = None
    """The secret to use for Exact Data Match datasets.

    This is not present in Custom Wordlists.
    """
